(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('rxjs'), require('rxjs/operators')) :
    typeof define === 'function' && define.amd ? define('reactive-entity-store', ['exports', 'rxjs', 'rxjs/operators'], factory) :
    (factory((global['reactive-entity-store'] = {}),global.rxjs,global.rxjs.operators));
}(this, (function (exports,rxjs,operators) { 'use strict';

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */
    var __assign = function () {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s)
                    if (Object.prototype.hasOwnProperty.call(s, p))
                        t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };
    function __read(o, n) {
        var m = typeof Symbol === "function" && o[Symbol.iterator];
        if (!m)
            return o;
        var i = m.call(o), r, ar = [], e;
        try {
            while ((n === void 0 || n-- > 0) && !(r = i.next()).done)
                ar.push(r.value);
        }
        catch (error) {
            e = { error: error };
        }
        finally {
            try {
                if (r && !r.done && (m = i["return"]))
                    m.call(i);
            }
            finally {
                if (e)
                    throw e.error;
            }
        }
        return ar;
    }

    function createUniqueId(object, length) {
        if (length === void 0) {
            length = 10;
        }
        var id;
        for (var i = 0; i === 0 || object.hasOwnProperty(id); i++) {
            id = Math.random().toFixed(36).substring(2, 2 + length);
        }
        return id;
    }

    function addOne(store, entity) {
        if (!entity) {
            return store;
        }
        addToStore(store, entity);
        return store;
    }
    function addAll(payload) {
        return payload.reduce(function (store, entity) {
            addToStore(store, entity);
            return store;
        }, { entities: {}, ids: [] });
    }
    function addMany(store, payload) {
        payload.forEach(function (entity) {
            addToStore(store, entity);
        });
        return store;
    }
    function removeOne(store, id) {
        if (store.entities[id]) {
            delete store.entities[id];
            store.ids.splice(store.ids.indexOf(id), 1);
        }
        return store;
    }
    function removeAll() {
        return { entities: {}, ids: [] };
    }
    function updateOne(store, update) {
        var entity = store.entities[update.id];
        store.entities[update.id] = __assign({}, entity, update.payload);
        return store;
    }
    function safeId(entity, entities) {
        return entity['id'] ? encodeURIComponent(entity['id']) : createUniqueId(entities);
    }
    function addToStore(store, entity) {
        var id = safeId(entity, store.entities);
        if (store.entities[id]) {
            return store;
        }
        delete entity['id'];
        store.entities[id] = entity;
        store.ids.push(id);
        return store;
    }

    /**
     * Reactive crud store
     *
     * This store targets the creation of a very simple store
     * to keep your entities in. It provides modification functions
     * for adding, adding all, removing one by id, remove all,
     * update one.
     *
     * The read functions return observables to read the stored
     * entities in an object or array way. Also you can get a
     * single entity by an id.
     *
     * ## Example
     *
     * ```ts
     * const numberStore = new Store<number>();
     * numberStore.getAll().subscribe(console.log);
     * numberStore.add(8);
     * numberStore.add(3);
     * numberStore.removeAll();
     * // logs ---> [8] ---> [8, 3] ---> []
     * ```
     */
    var Store = /** @class */ (function () {
        function Store() {
            this.actions$ = new rxjs.Subject();
            this.store$ = this.createStore();
            this.entities$ = this.store$.pipe(operators.pluck('entities'));
        }
        Store.prototype.createStore = function () {
            return this.actions$.pipe(operators.scan(function (store, action) {
                switch (action.type) {
                    case 'add': return addOne(store, action.payload);
                    case 'addMany': return addMany(store, action.payload);
                    case 'addAll': return addAll(action.payload);
                    case 'update': return updateOne(store, action.payload);
                    case 'remove': return removeOne(store, action.payload);
                    case 'removeAll': return removeAll();
                }
            }, { entities: {}, ids: [] }), operators.startWith({ entities: {}, ids: [] }), operators.share());
        };
        Store.prototype.add = function (entity) { this.actions$.next({ type: 'add', payload: entity }); };
        Store.prototype.addMany = function (entities) { this.actions$.next({ type: 'addMany', payload: entities }); };
        Store.prototype.addAll = function (entities) { this.actions$.next({ type: 'addAll', payload: entities }); };
        Store.prototype.update = function (changes) { this.actions$.next({ type: 'update', payload: changes }); };
        Store.prototype.remove = function (id) { this.actions$.next({ type: 'remove', payload: id }); };
        Store.prototype.removeAll = function () { this.actions$.next({ type: 'removeAll' }); };
        /**
         * Get one entity filtered by id
         * @param id
         */
        Store.prototype.getOne = function (id) {
            return this.entities$.pipe(operators.pluck(id), operators.map(function (entity) { return (entity ? __assign({ id: id }, entity) : undefined); }));
        };
        /**
         * Get one item by a maybe changing variable. This for example can be useful
         * when getting data by router paramters
         * @param id$
         */
        Store.prototype.getOneDynamic = function (id$) {
            var _this = this;
            return id$.pipe(operators.switchMap(function (id) { return _this.getOne(id); }));
        };
        /**
         * Get all entities in an array including the id property
         */
        Store.prototype.getAll = function () {
            return this.entities$.pipe(operators.map(function (entities) {
                return Object.entries(entities).map(function (_a) {
                    var _b = __read(_a, 2), id = _b[0], entity = _b[1];
                    return (__assign({ id: id }, entity));
                });
            }));
        };
        /**
         * Get all entities as an object with keys
         */
        Store.prototype.getEntities = function () {
            return this.entities$;
        };
        /**
         * Return all ids
         */
        Store.prototype.getIds = function () {
            return this.store$.pipe(operators.pluck('ids'));
        };
        return Store;
    }());

    /**
     * Generated bundle index. Do not edit.
     */

    exports.Store = Store;

    Object.defineProperty(exports, '__esModule', { value: true });

})));

//# sourceMappingURL=reactive-entity-store.umd.js.map